@@ -44,6 +44,7 @@
 #include "extdbdata.h"
 #include "language.h"
 #include <set>
+#include "codelite_exports.h"
 
 #ifdef USE_TRACE
 #include <wx/stopwatch.h>
@@ -59,12 +60,15 @@ class IProcess;
 // the workspcae retag
 #define USE_PARSER_TREAD_FOR_RETAGGING_WORKSPACE 1
 
+// BUG#3082954
+#define MAX_TIP_LINE_SIZE 100
+
 #define TagsGlobal    0
 #define TagsGlobalGTK 1
 
 // send this event whenever the a tags file needs to be updated
-extern const wxEventType wxEVT_UPDATE_FILETREE_EVENT;
-extern const wxEventType wxEVT_TAGS_DB_UPGRADE;
+extern WXDLLIMPEXP_CL const wxEventType wxEVT_UPDATE_FILETREE_EVENT;
+extern WXDLLIMPEXP_CL const wxEventType wxEVT_TAGS_DB_UPGRADE;
 
 struct DoxygenComment {
 	wxString name;
@@ -121,10 +125,10 @@ class ITagsStorage;
  * @author Eran
  *
  */
-class TagsManager : public wxEvtHandler
+class WXDLLIMPEXP_CL TagsManager : public wxEvtHandler
 {
 	// Members
-	friend class Singleton<TagsManager>;
+	friend class TagsManagerST;
 	friend class DirTraverser;
 	friend class Language;
 
@@ -753,6 +757,21 @@ public:
 	 */
 	bool GetDerivationList(const wxString &path, std::vector<wxString> &derivationList, std::set<wxString> &scannedInherits);
 
+	/**
+	 * @brief return true if the file is binary (by searching for NULL chars)
+	 * @param filepath file to examine
+	 * @return return true if the file is binary
+	 */
+	bool IsBinaryFile(const wxString &filepath);
+	
+	/**
+	 * @brief given an input string 'str', wrap the string so each line will
+	 * not be longer than MAX_TIP_LINE_SIZE bytes
+	 */
+	wxString WrapLines(const wxString &str);
+	
+	void GetVariables(const std::string &in, VariableList &li, const std::map<std::string, std::string> &ignoreMap, bool isUsedWithinFunc);
+	
 protected:
 	// provide a default handler for the wxEVT_UPDATE_FILETREE_EVENT event
 	void OnUpdateFileTreeEvent(wxCommandEvent &e);
@@ -779,6 +798,10 @@ protected:
 };
 
 /// create the singleton typedef
-typedef Singleton<TagsManager> TagsManagerST;
+class WXDLLIMPEXP_CL TagsManagerST {
+public:
+	static TagsManager* Get();
+	static void         Free();
+};
 
 #endif // CODELITE_CTAGS_MANAGER_H
