@@ -449,6 +449,7 @@ void BreakptMgr::DelAllBreakpoints()
 	// Delete all markers before clearing m_bps, otherwise we won't know which files they were in
 	DeleteAllBreakpointMarkers();
 	m_bps.clear();
+	m_pendingBreakpointsList.clear();	// Delete any pending bps too
 }
 
 // Toggle a breakpoint's enabled state
@@ -711,10 +712,7 @@ void BreakptMgr::BreakpointHit(int id)
 	}
 
 	BreakpointInfo bp = m_bps.at(index);
-	if (bp.commandlist.IsEmpty()) {
-		return;
-	}
-
+	if (! bp.commandlist.IsEmpty()) {
 	IDebugger *dbgr = DebuggerMgr::Get().GetActiveDebugger();
 	if (dbgr && dbgr->IsRunning()) {
 		// A likely command, presumably at the end of the command-list, is 'continue' or 'cont'
@@ -738,6 +736,18 @@ void BreakptMgr::BreakpointHit(int id)
 	}
 }
 
+	if (bp.bp_type == BP_type_tempbreak) {
+		// If this is a temporary bp, remove it from m_bps now it's been hit
+		// Otherwise it will be treated as a 'Pending' bp, the button will be displayed
+		// and, if clicked, the bp will be resurrected.
+		int index = FindBreakpointById(id, m_bps);
+		if (index != wxNOT_FOUND) {
+			m_bps.erase(m_bps.begin()+index);
+		}
+
+	}
+}
+
 /*------------------------------- Implementation -------------------------------*/
 
 // Get a breakpoint on this line. If multiple bps, ask the user to select
