@@ -38,6 +38,7 @@ QuickFindBar::QuickFindBar(wxWindow* parent, wxWindowID id)
 		: QuickFindBarBase(parent, id)
 		, m_sci(NULL)
 		, m_flags(0)
+		, m_lastTextPtr(NULL)
 {
 	Hide();
 	DoShowControls();
@@ -46,11 +47,9 @@ QuickFindBar::QuickFindBar(wxWindow* parent, wxWindowID id)
 	wxTheApp->Connect(wxID_COPY,      wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(QuickFindBar::OnCopy),      NULL, this);
 	wxTheApp->Connect(wxID_PASTE,     wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(QuickFindBar::OnPaste),     NULL, this);
 	wxTheApp->Connect(wxID_SELECTALL, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(QuickFindBar::OnSelectAll), NULL, this);
-
 	wxTheApp->Connect(wxID_COPY,      wxEVT_UPDATE_UI, wxUpdateUIEventHandler(QuickFindBar::OnEditUI), NULL, this);
 	wxTheApp->Connect(wxID_PASTE,     wxEVT_UPDATE_UI, wxUpdateUIEventHandler(QuickFindBar::OnEditUI), NULL, this);
 	wxTheApp->Connect(wxID_SELECTALL, wxEVT_UPDATE_UI, wxUpdateUIEventHandler(QuickFindBar::OnEditUI), NULL, this);
-
 	wxTheApp->Connect(XRCID("find_next"),              wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(QuickFindBar::OnFindNext),          NULL, this);
 	wxTheApp->Connect(XRCID("find_previous"),          wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(QuickFindBar::OnFindPrevious),      NULL, this);
 	wxTheApp->Connect(XRCID("find_next_at_caret"),     wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(QuickFindBar::OnFindNextCaret),     NULL, this);
@@ -63,14 +62,34 @@ bool QuickFindBar::Show(bool show)
 	return DoShow(show, wxEmptyString);
 }
 
+wchar_t* QuickFindBar::DoGetSearchStringPtr()
+{
+	wxString text = m_sci->GetText();
+	wchar_t *pinput (NULL);
+	if(m_lastText == text && m_lastTextPtr){
+		pinput = m_lastTextPtr;
+		
+	} else {
+		m_lastText    = text;
+#if wxVERSION_NUMBER >= 2900
+		m_lastTextPtr = const_cast<wchar_t*>(m_lastText.c_str().AsWChar());
+#else
+		m_lastTextPtr = const_cast<wchar_t*>(m_lastText.c_str());
+#endif
+		pinput        = m_lastTextPtr;
+	}
+	return pinput;
+}
+
 void QuickFindBar::DoSearch(bool fwd, bool incr)
 {
 	if (!m_sci || m_sci->GetLength() == 0 || m_findWhat->GetValue().IsEmpty())
 		return;
 
 	wxString find = m_findWhat->GetValue();
-	wxString text = m_sci->GetText();
-
+	wchar_t* pinput = DoGetSearchStringPtr();
+	if(!pinput)
+		return;
 	int start = -1, stop = -1;
 	m_sci->GetSelection(&start, &stop);
 
@@ -78,7 +97,7 @@ void QuickFindBar::DoSearch(bool fwd, bool incr)
 	int flags = m_flags | (fwd ? 0 : wxSD_SEARCH_BACKWARD);
 	int pos = 0, len = 0;
 
-	if (!StringFindReplacer::Search(text, offset, find, flags, pos, len)) {
+	if (!StringFindReplacer::Search(pinput, offset, find, flags, pos, len)) {
 
 		// wrap around and try again
 		wxString msg = fwd ? _("Reached end of document, continued from start") : _("Reached top of document, continued from bottom");
@@ -101,8 +120,8 @@ void QuickFindBar::DoSearch(bool fwd, bool incr)
 
 		if(res == wxID_OK) {
 
-			offset = fwd ? 0 : text.Len()-1;
-			if (!StringFindReplacer::Search(text, offset, find, flags, pos, len)) {
+			offset = fwd ? 0 : wxStrlen(pinput) - 1;
+			if (!StringFindReplacer::Search(pinput, offset, find, flags, pos, len)) {
 				m_findWhat->SetBackgroundColour(wxT("PINK"));
 				m_findWhat->Refresh();
 				return;
@@ -119,9 +138,13 @@ void QuickFindBar::DoSearch(bool fwd, bool incr)
 	m_findWhat->Refresh();
 	m_sci->SetSelection(pos, pos+len);
 
-	// Ensure that the found string is visible (e.g. its line isn't folded away)
+	// Ensure that the found string is visible (i.e. its line isn't folded away)
+	// and that the user can see it without having to scroll
 	int line = m_sci->LineFromPosition(pos);
-	if ( line >= 0 ) m_sci->EnsureVisible(line);
+	if ( line >= 0 ) {
+		m_sci->EnsureVisible(line);
+		m_sci->EnsureCaretVisible();
+	}
 }
 
 void QuickFindBar::OnHide(wxCommandEvent &e)
@@ -555,7 +578,10 @@ void QuickFindBar::DoMarkAll()
 	flags &= ~ wxSD_SEARCH_BACKWARD;
 	int offset(0);
 
-	wxString txt = m_sci->GetText();
+	wchar_t* pinput = DoGetSearchStringPtr();
+	if(!pinput)
+		return;
+	
 	int fixed_offset(0);
 
 	editor->DelAllMarkers();
@@ -563,7 +589,7 @@ void QuickFindBar::DoMarkAll()
 	// set the active indicator to be 1
 	editor->SetIndicatorCurrent(1);
 
-	while ( StringFindReplacer::Search(txt, offset, findWhat, flags, pos, match_len) ) {
+	while ( StringFindReplacer::Search(pinput, offset, findWhat, flags, pos, match_len) ) {
 		editor->MarkerAdd(editor->LineFromPosition(fixed_offset + pos), smt_bookmark);
 
 		// add indicator as well
@@ -624,3 +650,17 @@ void QuickFindBar::OnHighlightMatchesUI(wxUpdateUIEvent& event)
 		}
 	}
 }
+
+QuickFindBar::~QuickFindBar()
+{
+	wxTheApp->Disconnect(wxID_COPY,      wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(QuickFindBar::OnCopy),      NULL, this);
+	wxTheApp->Disconnect(wxID_PASTE,     wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(QuickFindBar::OnPaste),     NULL, this);
+	wxTheApp->Disconnect(wxID_SELECTALL, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(QuickFindBar::OnSelectAll), NULL, this);
+	wxTheApp->Disconnect(wxID_COPY,      wxEVT_UPDATE_UI, wxUpdateUIEventHandler(QuickFindBar::OnEditUI), NULL, this);
+	wxTheApp->Disconnect(wxID_PASTE,     wxEVT_UPDATE_UI, wxUpdateUIEventHandler(QuickFindBar::OnEditUI), NULL, this);
+	wxTheApp->Disconnect(wxID_SELECTALL, wxEVT_UPDATE_UI, wxUpdateUIEventHandler(QuickFindBar::OnEditUI), NULL, this);
+	wxTheApp->Disconnect(XRCID("find_next"),              wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(QuickFindBar::OnFindNext),          NULL, this);
+	wxTheApp->Disconnect(XRCID("find_previous"),          wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(QuickFindBar::OnFindPrevious),      NULL, this);
+	wxTheApp->Disconnect(XRCID("find_next_at_caret"),     wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(QuickFindBar::OnFindNextCaret),     NULL, this);
+	wxTheApp->Disconnect(XRCID("find_previous_at_caret"), wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(QuickFindBar::OnFindPreviousCaret), NULL, this);
+}
